;
;    ********************************
;    *  Kinder-Bastelprojekt Sirene *
;    ********************************
;
; Created 23.2.2013
;     23. 2.13: Full function, Demo Version 
;      2. 3.13: AF-PA, Sounds improved, 1-Minute-Alive-Warning, 
;               This Level used for 4.3.13 P12 Kinderbasteln 
;      7. 3.13: Added target option ATtiny13   
;
; Luft auf ATtiny12 oder ATtiny13, erfordert aber unterschiedliche FUSE-Programmierung
;   und .hex-Files!
;
.equ	TINY12 =0	; UNBEDINGT auf 1 setzen fr ATtiny12, auf 0 fr ATtiny13
;
; **** Programmierung eines ATtiny12:
; Fuses fr ATtiny12 werden wie folgt programmiert:
;	Brown-Out detection at VCC=2,7 V
;	BODEN:    Hkchen
;	SPIEN:    Hkchen
;	RSTDISBL: KEIN Hkchen
; 	CKSEL:    Internal RCOscillator (1,2 MHz)
;	Fuse-Pattern 0x12
;
; NACH dem Beschreiben des Flash mit dem Programm muss noch das Oscillator Calibration 
; Byte OSCCAL gelesen und in die letzte Flash Location $03FF geschrieben werden!
;
; **** Programmierung eines ATtiny13:
; Fuses fr ATtiny13 werden wie folgt programmiert:
;	Brown-Out detection at VCC=2,7 V
;	SPIEN:     Hkchen
;	CKDIV8:    Hkchen
; 	SUT_CKSEL: Int. RC Osc9,6MHz; Stat-up time 14CK + 64 ms
;	Fuse-Pattern High 0xFB Low 0x6A
; Das Oscillator Calibration Byte wird vom Prozessor selbstttig geladen 
;   (muss also nicht, wie beim ATtiny12, vom Programmer ins Flash und vom Programm
;    selber weiter ins OSCCAL kopiert werden). Das macht das programmieren einfacher.
;
;
	;***** Specify Device
.device ATtiny12	; Braucht nicht gendert werden fr ATtiny13! 	
	;
	;***** I/O Register Definitions
	;
.equ    TCCR0   =$33
.equ    TCNT0   =$32
.equ	OSCCAL	=$31
.equ    PORTB   =$18
.equ    DDRB    =$17
.equ    PINB    =$16
	;
	;**** Register Useage (the ATtiny12 has no RAM)
;
; 	R0 	Reserved for Flash Access
;
;	R2	Alive Timer - to generate alive beep every 60 Seconds
; 	R16     Work register for Main
;	R17	Second work register
;	R18	Third work register
;	R20 	Sound Selector, stepped 0..5 with button on PB3
;	R21	Potentiometer value, 90..250, read by POTIN
;	R25	Parameter in msec for DELNMS, DELNUS
;	R26	Work register for DELMS, DELNUS, POTIN
;	R27	Work register for SOUND
;	R28	Work register for NBEEP
.DEF    ZL      =r30    ; Used for Flash access
.DEF    ZH      =r31	; dto 
	;
	;**** Code starts here *****************************************************
	.CSEG
        .ORG 0
        ;
	; ++++ Interrupt Vectors
	;
        RJMP    START           ;  1 Reset   
        RJMP    START           ;  2 
        RJMP    START           ;  3  
        RJMP    START           ;  4 Timer 0 Overflow
        RJMP    START           ;  5 
        ;
START:	; ++++ Initialize OSCCAL value for ATtiny12 -----------------------------
	.IF TINY12==1
	ldi	ZH,$03
	ldi	ZL,$FF
	lpm			; Get Calibration Byte from Flash to R0
	out	OSCCAL,R0	; and write to OSCCAL register
	nop			; Give Clock time to settle
	nop
	.ENDIF
	;
  	; ++++ Initialize Port B Bit 0 as Output Poti, High to load C
	;			 Bit 1 as Input Unused w Pull-Up
	;			 Bit 2 as Output Speaker, Low
	;			 Bit 3,4 as Inputs Button w Pull-Up
	ldi     R16,$05
        out     DDRB,R16
        ldi	R16,$1B	
        out	PORTB,R16
        ;
	;++++ Intialize Timer/Counter 0 as 1 Millisecond Clock 
	;		(1,2 Mhz/1024 is actually 850us)
	ldi	R16,$05		; Set Clock Divider to 1024
	out	TCCR0,R16
	;
	;
	clr	R20		; Sound selector to 0
	rcall	NBEEP		; Ready Beep
	;
	; **** Main Loop ****************************************************
	;
	; **** Wait for any button, Generate Alive Beep
MAIN:	clr	R2		; Reset Alive Timer
MA1:	sbis	PINB,3		; Select Button PB3?
	rjmp	SELECT
	sbis	PINB,4		; Play Button PB4?
	rjmp	PLAY
	;
	ldi	R25,255		; Wait 255 * 0,85 = 216 msec		
	rcall	DELNMS
	inc	R2		; Increment Alive Timer
	brne	MA1		; Up to 256 * 216 msec = 55 sec
	;
	ldi	R24,30		; Generate double tone for Alive indication
	ldi	R25,40
	rcall	BEEP
	ldi	R24,25
	rcall	BEEP
	rjmp	MAIN
	;
	; Handle Select Button
SELECT:	inc	R20		; Increment Selector up to 5
	cpi	R20,6
	brne	SEL1
	clr	R20
SEL1:	rcall	NBEEP		; Ouput new value as n+1 beeps
	rjmp	MAIN	
	;
	; Handle Play Button
PLAY:	rcall	POTIN		; Get Potentiometer value
PL00:	cpi	R20,0		; Fork to 6 different PLAYs 
	brne	PL01
	rjmp	PLAY1
PL01:	cpi	R20,1
	brne	PL02
	rjmp	PLAY2
PL02:	cpi	R20,2
	brne	PL03
	rjmp	PLAY3
PL03:	cpi	R20,3
	brne	PL04
	rjmp	PLAY4
PL04:	cpi	R20,4
	brne	PL05
	rjmp	PLAY5
PL05:	rjmp	PLAY6
	;
	; PLAY1: Variable frequency contiuous tone
PLAY1:	mov	R24,R21		; Use variable as beep frequency
	ldi	R25,40		; Fixed Beep length, no pause
P11:	rcall	BEEP
	sbis	PINB,4		; Play Button PB4 still down?
	rjmp	P11		; Then continue to Beep
	rjmp	MAIN		; Done
	;
	; PLAY2: Variable frequency intermittend tone
PLAY2:	mov	R24,R21		; Use variable as beep frequency
	mov	R25,R21		; and beep length
	rcall	BEEP
	lsr	R25
	rcall	DELNMS		; Pause length equals 1/2 beep length
	rjmp	MAIN		; Done
	;
	; PLAY3: TaTTaTaa 
PLAY3:	mov	R25,R21		; Use variable as speed
	ldi	R24,100		; Ta
	rcall	BEEP
	ldi	R24,60		; T
	rcall	BEEP
	rcall	BEEP
	ldi	R24,100		; Ta
	rcall	BEEP
	ldi	R24,150		; Taa
	rcall	BEEP
	rcall	BEEP
	rjmp	MAIN
	;
	; PLAY4: Upward Tone
PLAY4:	mov	R24,R21		; Use variable as start frequency
	ldi	R25,10		; Beep length 10 msec per tone
P41:	rcall	BEEP
	dec	R24
	cpi	R24,20		; Increase frequency to 3 kHz
	brne	P41
	ldi	R25,100		; 100 msec pause
	rcall	DELNMS		
	rjmp	MAIN		; Done
	;
	; PLAY5: Fast Upward + Downward Tone
PLAY5:	mov	R16,R21		; Use variable as beep start frequency
	mov	R24,R16
	ldi	R25,3 		; Beep length 3 msec per tone
P50:	rcall	BEEP
	dec	R24
	cpi	R24,20		; Increase frequency to 3 kHz
	brne	P50
P51:	rcall	BEEP		; And back down 
	inc	R24
	cp	R24,R16		; Decrease frequency to start frequency
	brne	P51
	rjmp	MAIN		; Done
	;
	; PLAY6: Morse MORSE
PLAY6:	ldi	R24,80		; 800 Hz
	mov	R25,R21		; Use variable as dot length (40-120 Bpm)
	rcall 	DASH
	rcall	DASH
	rcall	BREAK2
	rcall	DASH
	rcall	DASH
	rcall	DASH
	rcall	BREAK2
	rcall	DOT
	rcall	DASH
	rcall 	DOT
	rcall	BREAK2
	rcall	DOT
	rcall	DOT
	rcall	DOT
	rcall	BREAK2
	rcall	DOT
	rcall	BREAK2
	rcall	BREAK2
	rcall	BREAK2
	rjmp	MAIN
	;
        ;
	;**** Subroutines ****************************************************
	;
	; Generate Sound, Length in msec (1..250) in R25, Frequency in R24
	; Frequency is 1 / (R24 * 10 usec)  => 255 = 250 Hz, 65 = 1 kHz, 0 = Off
BEEP:	clr	R27		; Reset msec Clock
	out	TCNT0,R27
	;
SO1:	tst	R24		; No sound if R24 = 0
	breq	SO3		
SO2:	sbi	PORTB,2		; Speaker to H
	rcall	DELUS		; Wait 1/2 cycle
	cbi	PORTB,2		; Speaker to L
	rcall	DELUS		; Wait 1/2 cycle
SO3:	in	R27,TCNT0	; Check Clock
	cp	R27,R25
	brcs	SO1		; Loop until desired sound length
	ret
	;
	; Morse Dot, Length in ms in R25
DOT:	rcall	BEEP
	rcall	DELNMS
	ret
	;
	; Morse Dash, Length in ms in R25
DASH:	rcall	BEEP
	rcall	BEEP
	rcall	BEEP
	rcall	DELNMS
	ret
	;
	; Morse Break, 2 Dots long
BREAK2:	rcall	DELNMS
	rcall	DELNMS
	rcall	POTIN		; Update morse speed for next char
	mov	R25,R21
	ret
	;
	; Indicate Value of Sound Selector R20 as n+1 Beeps
NBEEP:	mov	R28,R20		; Generate n+1 Beeps
NB1:	ldi	R25,150		; 2 Beeps per Second
	ldi	R24,250		; 250 Hz
	rcall	BEEP
	rcall	DELNMS
	tst	R28
	breq	NB9
	dec	R28
	rjmp	NB1
NB9:	rcall	DELNMS
	rcall	DELNMS
	ret
	;
	; Read Potentiometer Value on PB0 as 40..250 into R21
POTIN:	cbi	DDRB,0		; Switch PB0 from Output H to Input w/o Pullup
	cbi	PORTB,0
	clr	R21
PI1:	ldi	R26,7		; Adjust loop length to RC time constant
PI11:	dec	R26
	brne	PI11
	inc	R21		; Count time until C unloaded
	cpi	R21,250		; Limit Counter to max value 250
	breq	PI2
	sbic	PINB,0		; Test C, leave loop when unloaded
	rjmp	PI1
	;
PI2:	sbi	DDRB,0		; Switch PB0 back to Output H to reload C
	sbi	PORTB,0
	cpi	R21,40		; Min value is 40
	brcc	PI3
	ldi	R21,40
	;
PI3:	ret
	;
	;
	; ** n * 1msec Delay, n in R25 (1..250), uses Timer 0
DELNMS:	clr	R27		; Reset msec Clock
	out	TCNT0,R27
DNMS1:	in	R27,TCNT0	; Wait until clock = desired value
	cp	R27,R25
	brne	DNMS1
	ret
	;
	;
	; ** n * 10 usec Delay, n in R24
DELUS:	mov	R26,R24 
DUS1:	dec	R26	; 12 cycle loop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	brne	DUS1
	ret
	;
;**************************** The End ***************************** 
